@@ -54,6 +54,13 @@
 */
 #define DEFAULT_SSL_ROOT_CERTS_LOCATION NULL
 
+/*
+ * By default we let OpenSSL choose the ciphers we wnat to use.
+ * But if you were really bent on overruling that, you could do it here.
+ * If you change this, you are expected to know what you are doing.
+ */
+#define DEFAULT_SSL_CIPHERS NULL
+
 /*
  * This is an experimental feature to thwart infinite recursion.  It is not
  * very sophisticated so it's not turned on by default.  In the future I 
@@ -264,6 +271,7 @@
  * remove them.  They are default values for variables and are required for
  * proper compilation.
  */
+#define DEFAULT_ACCEPT_INVALID_SSL_CERT 1
 #define DEFAULT_ALLOW_C1_CHARS 0
 #define DEFAULT_ALT_CHARSET 1
 #define DEFAULT_ALWAYS_SPLIT_BIGGEST 1
@@ -367,7 +375,7 @@
 #define DEFAULT_STATUS_HALFOP "%"
 #define DEFAULT_STATUS_HOLD " Held: "
 #define DEFAULT_STATUS_HOLD_LINES "%B"
-#define DEFAULT_STATUS_HOLDMODE " (Hold) "
+#define DEFAULT_STATUS_HOLDMODE " (Hold)"
 #define DEFAULT_STATUS_INSERT ""
 #define DEFAULT_STATUS_MAIL " (Mail: %M)"
 #define DEFAULT_STATUS_MODE " (+%+)"
