@@ -76,6 +76,7 @@ enum HOOK_TYPES {
 	PUBLIC_OTHER_LIST,
 	RAW_IRC_LIST,
 	RAW_IRC_BYTES_LIST,
+	RECONNECT_REQUIRED_LIST,
 	REDIRECT_LIST,
 	SEND_ACTION_LIST,
 	SEND_CTCP_LIST,
@@ -88,6 +89,7 @@ enum HOOK_TYPES {
 	SERVER_ESTABLISHED_LIST,
 	SERVER_LOST_LIST,
 	SERVER_NOTICE_LIST,
+	SERVER_SSL_EVAL_LIST,
 	SERVER_STATE_LIST,
 	SERVER_STATUS_LIST,
 	SET_LIST,
