@@ -75,7 +75,8 @@ int	ascii_strnicmp 		(const unsigned char *, const unsigned char *, size_t);
 int	ascii_stricmp 		(const unsigned char *, const unsigned char *);
 int	rfc1459_strnicmp 	(const unsigned char *, const unsigned char *, size_t);
 int	rfc1459_stricmp 	(const unsigned char *, const unsigned char *);
-char *  uuid4_generate (void);
+char *  uuid4_generate 		(void);
+char *	uuid4_generate_no_dashes	(void);
 
 
 void	really_free 		(int);
@@ -326,7 +327,8 @@ extern	int     is_iso2022_jp (const unsigned char *buffer);
 extern	void    create_utf8_locale (void);
 extern	int	mkupper_l	(int);
 extern	int	mklower_l	(int);
-extern	int	sanity_check_encoding (const char *);
+/* extern	int	sanity_check_encoding (const char *); */
+extern	unsigned char *  cp437_to_utf8 (const unsigned char *input, size_t inputlen, size_t *destlen);
 
 /******* recode.c ******/
 extern	void		init_recodings	(void);
