@@ -72,6 +72,8 @@
 #define NEWIO_RECV	  5
 #define NEWIO_NULL	  6
 #define NEWIO_SSL_CONNECT 7
+#define NEWIO_PASSTHROUGH_READ 8
+#define NEWIO_PASSTHROUGH_WRITE 9
 
 #define IO_BUFFER_SIZE 8192
 
