@@ -7,7 +7,7 @@
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
  *
- * @(#)$Id: output.h,v 1.4 2003/10/10 06:22:38 jnelson Exp $
+ * @(#)$Id: output.h,v 1.10 2008/11/28 16:28:03 jnelson Exp $
  */
 
 #ifndef __output_h__
@@ -22,11 +22,13 @@ struct ScreenStru;
 	void	say 			(const char *, ...) __A(1);
 	void	yell 			(const char *, ...) __A(1);
 	void    privileged_yell 	(const char *, ...) __A(1);
+	void	syserr			(int, const char *, ...) __A(2);
 
-	void	error			(const char *, ...) __A(1);
+	void	my_error		(const char *, ...) __A(1);
 	SIGNAL_HANDLER(sig_refresh_screen);
-	void	refresh_a_screen 	(struct ScreenStru *);
-	void	refresh_screen 		(char, char *);
+	void	redraw_all_screens 	(void);
+	BUILT_IN_KEYBINDING(refresh_screen);
 	int	init_screen 		(void);
 	void   	file_put_it 		(FILE *fp, const char *format, ...);
+
 #endif /* _OUTPUT_H_ */
