@@ -17,7 +17,8 @@ enum VAR_TYPES {
 extern
 #endif
 
-int	ALLOW_C1_CHARS_VAR,
+int	ACCEPT_INVALID_SSL_CERT_VAR,
+	ALLOW_C1_CHARS_VAR,
 	ALWAYS_SPLIT_BIGGEST_VAR,
 	BANNER_VAR,
 	BANNER_EXPAND_VAR,
@@ -108,6 +109,7 @@ int	ALLOW_C1_CHARS_VAR,
 	SHOW_CHANNEL_NAMES_VAR,
 	SHOW_NUMERICS_VAR,
 	SHOW_STATUS_ALL_VAR,
+	SSL_CIPHERS_VAR,
 	SSL_ROOT_CERTS_LOCATION_VAR,
 	STATUS_AWAY_VAR,
 	STATUS_CHANNEL_VAR,
