@@ -1,8 +1,7 @@
-/* $EPIC: debug.c,v 1.13 2006/11/16 23:44:19 jnelson Exp $ */
 /*
  * debug.c -- controll the values of x_debug.
  *
- * Copyright  1997, 2002 EPIC Software Labs
+ * Copyright 1997, 2006 EPIC Software Labs
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,9 +33,14 @@
 #include "irc.h"
 #include "ircaux.h"
 #include "output.h"
-#include "functions.h"
+#include "reg.h"
+#include "commands.h"
 
+#if 0
+unsigned long x_debug = DEBUG_SERVER_CONNECT;
+#else
 unsigned long x_debug = 0;
+#endif
 
 struct debug_opts
 {
@@ -47,7 +51,7 @@ struct debug_opts
 static struct debug_opts opts[] = 
 {
 	{ "LOCAL_VARS",		DEBUG_LOCAL_VARS },
-	{ "ALIAS",		DEBUG_ALIAS },
+	{ "ALIAS",		0 },
 	{ "CHANNELS",		DEBUG_CHANNELS },
 	{ "CTCPS",		DEBUG_CTCPS },
 	{ "DCC_SEARCH",		DEBUG_DCC_SEARCH },
@@ -55,24 +59,30 @@ static struct debug_opts opts[] =
 	{ "INBOUND",		DEBUG_INBOUND },
 	{ "DCC_XMIT",		DEBUG_DCC_XMIT },
 	{ "WAITS",		DEBUG_WAITS },
-	{ "MEMORY",		DEBUG_MEMORY },
+	{ "MEMORY",		0 },
 	{ "SERVER_CONNECT",	DEBUG_SERVER_CONNECT },
 	{ "CRASH",		DEBUG_CRASH },
-	{ "COLOR",		DEBUG_COLOR },
+	{ "NO_COLOR",		DEBUG_NO_COLOR },
 	{ "NOTIFY",		DEBUG_NOTIFY },
 	{ "REGEX",		DEBUG_REGEX },
 	{ "REGEX_DEBUG",	DEBUG_REGEX_DEBUG },
 	{ "BROKEN_CLOCK",	DEBUG_BROKEN_CLOCK },
 	{ "UNKNOWN",		DEBUG_UNKNOWN },
-	{ "BOLD_HELPER",	DEBUG_BOLD_HELPER },
-	{ "NEW_MATH",		DEBUG_NEW_MATH },
+	{ "BOLD_HELPER",	0 },
+	{ "NEW_MATH",		0 },
 	{ "NEW_MATH_DEBUG",	DEBUG_NEW_MATH_DEBUG },
-	{ "AUTOKEY",		DEBUG_AUTOKEY },
+	{ "AUTOKEY",		0 },
 	{ "EXTRACTW",		DEBUG_EXTRACTW },
 	{ "SLASH_HACK",		DEBUG_SLASH_HACK },
 	{ "LASTLOG",		DEBUG_LASTLOG },
 	{ "SSL",		DEBUG_SSL },
 	{ "EXTRACTW_DEBUG",	DEBUG_EXTRACTW_DEBUG },
+	{ "MESSAGE_FROM",	DEBUG_MESSAGE_FROM },
+	{ "WHO_QUEUE",		DEBUG_WHO_QUEUE },
+	{ "OLD_MATH",		0 },
+	{ "UNICODE",		DEBUG_UNICODE },
+	{ "DWORD",        	DEBUG_DWORD },
+	{ "RECODE",		DEBUG_RECODE },
 	{ "ALL",		~0},
 	{ NULL,			0 },
 };
@@ -81,9 +91,10 @@ static struct debug_opts opts[] =
 
 BUILT_IN_COMMAND(xdebugcmd)
 {
-	int cnt;
-	int rem = 0;
-	char *this_arg;
+	int 	cnt;
+	int 	rem = 0;
+	char *	this_arg;
+	int	original_xdebug = x_debug;
 
 	if (!args || !*args)
 	{
@@ -102,7 +113,7 @@ BUILT_IN_COMMAND(xdebugcmd)
 		return;
 	}
 
-	while (args && *args)
+	while (args && *args && *args != '{')
 	{
 		this_arg = upper(next_arg(args, &args));
 		if (*this_arg == '-')
@@ -112,10 +123,11 @@ BUILT_IN_COMMAND(xdebugcmd)
 
 		for (cnt = 0; opts[cnt].command; cnt++)
 		{
-			if (!strncmp(this_arg, opts[cnt].command, strlen(this_arg)))
+			if (!strncmp(this_arg, opts[cnt].command, 
+						strlen(this_arg)))
 			{
 				if (rem)
-					x_debug &= ~(opts[cnt].flag);
+					x_debug &= ~opts[cnt].flag;
 				else
 					x_debug |= opts[cnt].flag;
 				break;
@@ -124,12 +136,18 @@ BUILT_IN_COMMAND(xdebugcmd)
 		if (!opts[cnt].command)
 			say("Unrecognized XDEBUG option '%s'", this_arg);
 	}
+
+	if (args && *args && *args == '{')
+	{
+		runcmds(args, subargs);
+		x_debug = original_xdebug;
+	}
 }
 
 char *	function_xdebug (char *word)
 {
-	char	*ret = NULL;
-	char	*mask = NULL;
+	char	*ret = NULL, *free_str = NULL;
+	const char	*mask = NULL;
 	int	cnt;
 	size_t	clue = 0;
 
@@ -150,5 +168,10 @@ char *	function_xdebug (char *word)
 		malloc_strcat_c(&ret, opts[cnt].command, &clue);
 	}
 
-	RETURN_MSTR(ret);
+	if (word && *word)
+		malloc_strcat_wordlist_c(&ret, space,
+			free_str = function_xdebug(word), &clue);
+	new_free(&free_str);
+	malloc_strcat_c(&ret, "", &clue);
+	return ret;
 }
