@@ -206,6 +206,7 @@ Hookables hook_function_templates[] =
 	{ "PUBLIC_OTHER",	NULL,	3,	0,	0,	NULL, 0 },
 	{ "RAW_IRC",		NULL,	1,	0,	0,	NULL, 0 },
 	{ "RAW_IRC_BYTES",	NULL,	1,	0,	0,	NULL, 0 },
+	{ "RECONNECT_REQUIRED",	NULL,	1,	0,	0,	NULL, 0 },
 	{ "REDIRECT",		NULL,	2,	0,	HF_NORECURSE,	NULL, 0 },
 	{ "SEND_ACTION",	NULL,	2,	0,	HF_NORECURSE,	NULL, 0 },
 	{ "SEND_CTCP",		NULL,	3,	0,	HF_NORECURSE,	NULL, 0 },
@@ -218,13 +219,14 @@ Hookables hook_function_templates[] =
 	{ "SERVER_ESTABLISHED",	NULL,	2,	0,	0,	NULL, 0 },
 	{ "SERVER_LOST",	NULL,	2,	0,	0,	NULL, 0 },
 	{ "SERVER_NOTICE",	NULL,	1,	0,	0,	NULL, 0 },
+	{ "SERVER_SSL_EVAL",	NULL,	6,	0,	0,	NULL, 0 },
 	{ "SERVER_STATE",	NULL,	3,	0,	0,	NULL, 0 },
 	{ "SERVER_STATUS",	NULL,	3,	0,	0,	NULL, 0 },
 	{ "SET",		NULL,	2,	0,	0,	NULL, 0 },
 	{ "SIGNAL",		NULL,	1,	0,	0,	NULL, 0 },
 	{ "SIGNOFF",		NULL,	1,	0,	0,	NULL, 0 },
 	{ "SILENCE",		NULL,	2,	0,	0,	NULL, 0 },
-	{ "SSL_SERVER_CERT",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "SSL_SERVER_CERT",	NULL,	9,	0,	0,	NULL, 0 },
 	{ "STATUS_UPDATE",	NULL,	3,	0,	0,	NULL, 0 },
 	{ "SWITCH_CHANNELS",	NULL,	3,	0,	0,	NULL, 0 },
 	{ "SWITCH_QUERY",	NULL,	3,	0,	0,	NULL, 0 },
@@ -272,10 +274,9 @@ static void	initialize_hook_functions (void)
 {
 	int	i, b;
 	char *  p;
-	char *	buffer;
 
 	hook_functions = malloc(NUMBER_OF_LISTS * sizeof(Hookables));
-	p = buffer = malloc(4050);
+	p = malloc(4050);
 
 	for (i = 0; i < FIRST_NAMED_HOOK; i++, p += 4)
 	{
