@@ -1,8 +1,7 @@
-/* $EPIC: logfiles.c,v 1.37 2010/02/15 03:59:10 jnelson Exp $ */
 /*
  * logfiles.c - General purpose log files
  *
- * Copyright  2002 EPIC Software Labs
+ * Copyright 2002, 2012 EPIC Software Labs
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -75,6 +74,7 @@ struct Logfile {
 typedef struct Logfile Logfile;
 Logfile *logfiles = NULL;
 int	logref = 0;
+int	last_logref = -1;
 
 static Logfile *	new_logfile (void)
 {
@@ -93,6 +93,7 @@ static Logfile *	new_logfile (void)
 
 	log->next = NULL;
 	log->refnum = ++logref;
+	last_logref = log->refnum;
 	log->name = malloc_sprintf(NULL, "%d", log->refnum);
 	log->filename = NULL;
 	log->log = NULL;
@@ -123,7 +124,8 @@ static void	delete_logfile (Logfile *log)
 		for (prev = logfiles; prev; prev = prev->next)
 			if (prev->next == log)
 				break;
-		prev->next = log->next;
+		if (prev)
+			prev->next = log->next;
 	}
 
 	new_free(&log->name);
@@ -761,11 +763,12 @@ void	add_to_logs (long winref, int servref, const char *target, int level, const
 /*****************************************************************************/
 /* Used by function_logctl */
 /*
+ * $logctl(NEW)
  * $logctl(REFNUMS [ACTIVE|INACTIVE|ALL])
  * $logctl(REFNUM log-desc)
  * $logctl(ADD log-desc [target])
  * $logctl(DELETE log-desc [target])
- * $logctl(GET <refnum> [LIST]
+ * $logctl(GET <refnum> [LIST])
  * $logctl(SET <refnum> [ITEM] [VALUE])
  * $logctl(MATCH [pattern])
  * $logctl(PMATCH [pattern])
@@ -790,7 +793,12 @@ char *logctl	(char *input)
 	Logfile	*log;
 
 	GET_FUNC_ARG(listc, input);
-        if (!my_strnicmp(listc, "REFNUMS", 7)) {
+	if (!my_strnicmp(listc, "NEW", 3)) {
+		log = new_logfile();
+		RETURN_INT(log->refnum);
+	} else if (!my_strnicmp(listc, "LAST_CREATED", 12)) {
+		RETURN_INT(last_logref);
+	} else if (!my_strnicmp(listc, "REFNUMS", 7)) {
 		char *	retval = NULL;
 		int	active;
 
@@ -834,7 +842,7 @@ char *logctl	(char *input)
 			RETURN_EMPTY;
 
                 GET_FUNC_ARG(listc, input);
-                if (!my_strnicmp(listc, "REFNUM", 1)) {
+                if (!my_strnicmp(listc, "REFNUM", 3)) {
 			RETURN_INT(log->refnum);
                 } else if (!my_strnicmp(listc, "NAME", 3)) {
 			RETURN_STR(log->name);
@@ -937,3 +945,21 @@ void    logfiles_swap_winrefs (int oldref, int newref)
         }
 }
 
+void    logfiles_merge_winrefs (int oldref, int newref)
+{
+	Logfile *log;
+	int	i;
+
+	for (log = logfiles; log; log = log->next)
+        {
+		if (log->type != LOG_WINDOWS)
+			continue;
+
+		for (i = 0; i < MAX_TARGETS; i++)
+		{
+			if (log->refnums[i] == oldref)
+				log->refnums[i] = newref;
+		}
+        }
+}
+
