@@ -75,7 +75,7 @@ typedef	struct	myio_struct
 		held;
 	void	(*callback) (int vfd);
 	int	(*io_callback) (int vfd, int quiet);
-	int	(*failure_callback) (int channel, int error);
+	void	(*failure_callback) (int channel, int error);
 	int	quiet;
 	int	server;			/* For message routing */
 }           MyIO;
@@ -107,6 +107,7 @@ static int	unix_recv (int channel, int);
 static int	unix_accept (int channel, int);
 static int	unix_connect (int channel, int);
 static int	unix_close (int channel, int);
+static int	passthrough_event (int channel, int quiet);
 
 /* 
  * On systems where vfd != channel, you need these functions to 
@@ -581,13 +582,17 @@ int 	new_open (int channel, void (*callback) (int), int io_type, int quiet, int
 	else if (io_type == NEWIO_SSL_CONNECT)
 		ioe->io_callback = ssl_connect;
 #endif
+	else if (io_type == NEWIO_PASSTHROUGH_READ)
+		ioe->io_callback = passthrough_event;
+	else if (io_type == NEWIO_PASSTHROUGH_WRITE)
+		ioe->io_callback = passthrough_event;
 	else
 		panic(1, "New_open doesn't recognize io type %d", io_type);
 
 	ioe->callback = callback;
 	ioe->failure_callback = NULL;
 
-	if (io_type == NEWIO_CONNECT)
+	if (io_type == NEWIO_CONNECT || io_type == NEWIO_PASSTHROUGH_WRITE)
 	{
 		knowrite(vfd);
 		knoread(vfd);
@@ -911,6 +916,12 @@ static int	unix_connect (int channel, int quiet)
 	return (sizeof(int) + sizeof(SS)) * 2;
 }
 
+static int	passthrough_event (int channel, int quiet)
+{
+	/* We just push something so it's there. */
+	dgets_buffer(channel, "\n", 1);
+	return 1;
+}
 
 /*
  * Perform a synchronous i/o operation on a file descriptor.  
@@ -1012,6 +1023,23 @@ static  void    kwrite (int vfd) { FD_SET(CHANNEL(vfd), &writables); }
 static  void    knowrite (int vfd) { FD_CLR(CHANNEL(vfd), &writables); }
 static	void	kcleaned (int vfd) { return; }
 
+static	void	show_fd_set (fd_set *whatever)
+{
+	int	i;
+	char *	sigh = NULL;
+
+	for (i = 0; i <= global_max_channel; i++)
+	{
+		if (FD_ISSET(i, whatever))
+		{
+			malloc_strcat(&sigh, ltoa(i));
+			malloc_strcat(&sigh, " ");
+		}
+	}
+	yell("Waiting for fds: %s", sigh);
+	new_free(&sigh);
+}
+
 static	int	kdoit (Timeval *timeout)
 {
 	fd_set	working_rd, working_wd;
@@ -1020,6 +1048,7 @@ static	int	kdoit (Timeval *timeout)
 	working_rd = readables;
 	working_wd = writables;
 	errno = 0;
+	/* show_fd_set(&working_rd); */
 	retval = select(global_max_channel + 1, &working_rd, &working_wd, 
 			NULL, timeout);
 
