@@ -362,6 +362,7 @@ void 	init_variables_stage1 (void)
 {
 	var_bucket = new_bucket();
 
+	VAR(ACCEPT_INVALID_SSL_CERT,	BOOL, NULL)
 	VAR(ALLOW_C1_CHARS, 		BOOL, NULL)
 	VAR(ALWAYS_SPLIT_BIGGEST, 	BOOL, NULL)
 	VAR(BANNER, 			STR,  NULL)
@@ -464,6 +465,7 @@ void 	init_variables_stage1 (void)
 	VAR(SHOW_NUMERICS,              BOOL, NULL);
 	VAR(SHOW_STATUS_ALL,            BOOL, update_all_status_wrapper);
 #ifdef HAVE_SSL
+	VAR(SSL_CIPHERS,		STR, NULL);
 	VAR(SSL_ROOT_CERTS_LOCATION,    STR, set_ssl_root_certs_location);
 #endif
 	VAR(STATUS_AWAY,                STR,  build_status);
@@ -1115,6 +1117,10 @@ char *	make_string_var_bydata (const void *irc_variable)
 const 	VARIABLE *data;
 	char	*ret = (char *) 0;
 
+	/* XXX Not sure what better to do here */
+	if (!irc_variable)
+		return malloc_strdup(empty_string);
+
 	type = ((const IrcVariable *)irc_variable)->type;
 	data = (const VARIABLE *)(((const IrcVariable *)irc_variable)->data);
 
