@@ -1,4 +1,3 @@
-/* $EPIC: wserv.c,v 1.16 2010/01/01 01:45:46 jnelson Exp $ */
 /*
  * wserv.c -- A little program to act as a pipe between the ircII process
  * 	      and an xterm window or GNU screen.
@@ -9,7 +8,7 @@
  *
  * Copyright (c) 1992 Troy Rollo.
  * Copyright (c) 1993 Matthew Green.
- * Copyright  1995-2002 EPIC Software Labs
+ * Copyright 1995-2002 EPIC Software Labs
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -45,10 +44,12 @@
 #include "config.h"
 #include "irc_std.h"
 #include <sys/ioctl.h>
-#ifdef _ALL_SOURCE
-#include <termios.h>
+#ifdef HAVE_TERMIOS_H
+# include <termios.h>
 #else
-#include <sys/termios.h>
+# ifdef HAVE_SYS_TERMIOS_H
+#  include <sys/termios.h>
+# endif
 #endif
 
 
@@ -75,7 +76,6 @@ int	main (int argc, char **argv)
 	char * 	port;
 	char *	host;
 	char *	tmp;
-	int	t;
 	char	stuff[100];
 
 	my_signal(SIGHUP, SIG_IGN);
@@ -104,8 +104,13 @@ int	main (int argc, char **argv)
 	 * functions and some systems do not have snprintf!  This use of
 	 * sprintf is demonstrably safe.
 	 */
+#ifdef HAVE_SNPRINTF
+	snprintf(stuff, sizeof(stuff), "version=%d\n", CURRENT_WSERV_VERSION);
+#else
 	sprintf(stuff, "version=%d\n", CURRENT_WSERV_VERSION);
-	t = write(cmd, stuff, strlen(stuff));
+#endif
+	if (!write(cmd, stuff, strlen(stuff))) 
+		(void) 0;
 
 	/*
 	 * Next thing we take care of is to tell the parent client
@@ -118,10 +123,21 @@ int	main (int argc, char **argv)
 	 * attempts to use writev() have shown that linux does not have
 	 * an atomic writev() function (boo, hiss).  The parent client
 	 * does not take kindly to this data coming in multiple packets.
+	 *
+	 * Again, we use sprintf() here because we can't be guaranteed
+	 * that snprintf() is available in this program.  So I do the 
+	 * length check first, and fail if it would overrun.
 	 */
 	tmp = ttyname(0);
+	if (strlen(tmp) > 90)
+		my_exit(90);
+#ifdef HAVE_SNPRINTF
+	snprintf(stuff, sizeof(stuff), "tty=%s\n", tmp);
+#else
 	sprintf(stuff, "tty=%s\n", tmp);
-	t = write(cmd, stuff, strlen(stuff));
+#endif
+	if (!write(cmd, stuff, strlen(stuff))) 
+		(void) 0;
 
 	term_init();		/* Set up the xterm */
 	term_resize();		/* Figure out how big xterm is and tell epic */
@@ -150,14 +166,20 @@ int	main (int argc, char **argv)
 		if (FD_ISSET(0, &reads))
 		{
 			if ((nread = read(0, buffer, sizeof(buffer))) > 0)
-				write(data, buffer, nread);
+			{
+				if (!write(data, buffer, nread)) 
+					(void) 0;
+			}
 			else
 				my_exit(3);
 		}
 		if (FD_ISSET(data, &reads))
 		{
 			if ((nread = read(data, buffer, sizeof(buffer))) > 0)
-				write(0, buffer, nread);
+			{
+				if (!write(0, buffer, nread)) 
+					(void) 0;
+			}
 			else
 				my_exit(4);
 		}
@@ -170,7 +192,8 @@ static void 	ignore (int value)
 {
 	/* send a ^C */
 	char foo = 3;
-	write(data, &foo, 1);
+	if (!write(data, &foo, 1)) 
+		(void) 0;
 }
 
 static void	sigwinch_func (int value)
@@ -283,8 +306,13 @@ static void 	term_resize (void)
 	{
 		old_li = window.ws_row;
 		old_co = window.ws_col;
+#ifdef HAVE_SNPRINTF
+		snprintf(buffer, sizeof(buffer), "geom=%d %d\n", old_li, old_co);
+#else
 		sprintf(buffer, "geom=%d %d\n", old_li, old_co);
-		write(cmd, buffer, strlen(buffer));
+#endif
+		if (!write(cmd, buffer, strlen(buffer))) 
+			(void) 0;
 	}
 }
 
